@@ -17,6 +17,39 @@ pub struct CriticalSection(core::cell::UnsafeCell<portMUX_TYPE>);
 #[cfg(any(esp32c3, esp32s2))]
 pub struct CriticalSection(core::marker::PhantomData<*const ()>);
 
+#[inline(always)]
+#[link_section = ".iram1.interrupt_enter"]
+fn enter(cs: &CriticalSection) {
+    #[cfg(any(esp32c3, esp32s2))]
+    unsafe {
+        vPortEnterCritical();
+    }
+
+    #[cfg(all(esp_idf_version = "4.3", not(any(esp32c3, esp32s2))))]
+    unsafe {
+        vPortEnterCritical(cs.0.get());
+    }
+
+    #[cfg(all(not(esp_idf_version = "4.3"), not(any(esp32c3, esp32s2))))]
+    unsafe {
+        xPortEnterCriticalTimeout(cs.0.get(), portMUX_NO_TIMEOUT);
+    }
+}
+
+#[inline(always)]
+#[link_section = ".iram1.interrupt_exit"]
+fn exit(cs: &CriticalSection) {
+    #[cfg(any(esp32c3, esp32s2))]
+    unsafe {
+        vPortExitCritical();
+    }
+
+    #[cfg(not(any(esp32c3, esp32s2)))]
+    unsafe {
+        vPortExitCritical(cs.0.get());
+    }
+}
+
 impl CriticalSection {
     /// Constructs a new `CriticalSection` instance
     #[inline(always)]
@@ -55,20 +88,7 @@ impl CriticalSection {
     #[inline(always)]
     #[link_section = ".iram1.interrupt_cs_enter"]
     pub fn enter(&self) -> CriticalSectionGuard {
-        #[cfg(any(esp32c3, esp32s2))]
-        unsafe {
-            vPortEnterCritical()
-        };
-
-        #[cfg(all(esp_idf_version = "4.3", not(any(esp32c3, esp32s2))))]
-        unsafe {
-            vPortEnterCritical(self.0.get())
-        };
-
-        #[cfg(all(not(esp_idf_version = "4.3"), not(any(esp32c3, esp32s2))))]
-        unsafe {
-            xPortEnterCriticalTimeout(self.0.get(), portMUX_NO_TIMEOUT)
-        };
+        enter(self);
 
         CriticalSectionGuard(self)
     }
@@ -98,15 +118,7 @@ impl<'a> Drop for CriticalSectionGuard<'a> {
     #[inline(always)]
     #[link_section = ".iram1.interrupt_csg_drop"]
     fn drop(&mut self) {
-        #[cfg(any(esp32c3, esp32s2))]
-        unsafe {
-            vPortExitCritical()
-        };
-
-        #[cfg(not(any(esp32c3, esp32s2)))]
-        unsafe {
-            vPortExitCritical(self.0 .0.get())
-        };
+        exit(&self.0);
     }
 }
 
@@ -120,6 +132,51 @@ pub fn free<R>(f: impl FnOnce() -> R) -> R {
     f()
 }
 
+#[cfg(feature = "critical-section")]
+mod embassy_cs {
+    static CS: super::CriticalSection = super::CriticalSection::new();
+
+    struct EmbassyCriticalSectionImpl {}
+    critical_section::custom_impl!(EmbassyCriticalSectionImpl);
+
+    unsafe impl critical_section::Impl for EmbassyCriticalSectionImpl {
+        unsafe fn acquire() -> u8 {
+            super::enter(&CS);
+            return 1;
+        }
+
+        unsafe fn release(token: u8) {
+            if token != 0 {
+                super::exit(&CS);
+            }
+        }
+    }
+}
+
+#[cfg(feature = "embassy")]
+pub mod embassy {
+    pub enum CriticalSectionMutexKind {}
+    impl embassy::blocking_mutex::kind::MutexKind for CriticalSectionMutexKind {
+        type Mutex<T> = super::Mutex<T>;
+    }
+
+    impl<'a, T> embassy::blocking_mutex::Mutex for super::Mutex<T> {
+        type Data = T;
+
+        fn new(data: Self::Data) -> Self {
+            super::Mutex::new(data)
+        }
+
+        #[inline(always)]
+        #[link_section = ".iram1.interrupt_embmutex_lock"]
+        fn lock<R>(&self, f: impl FnOnce(&Self::Data) -> R) -> R {
+            let mut guard = super::Mutex::lock(self);
+
+            f(&mut guard)
+        }
+    }
+}
+
 /// A mutex based on critical sections
 pub struct Mutex<T> {
     cs: CriticalSection,
@@ -189,7 +246,7 @@ impl<'a, T> mutex_trait::Mutex for &'a Mutex<T> {
     }
 }
 
-#[cfg(feature = "embedded-svc-mutex")]
+#[cfg(feature = "embedded-svc")]
 impl<T> embedded_svc::mutex::Mutex for Mutex<T> {
     type Data = T;
 
